﻿* Encoding: UTF-8.
****************************************************************************************************************************************************
*Keep Calm and Learn Multilevel Logistic Modeling: A Simplified Three-Step Procedure for Beginners Using SPSS, Stata, and R*
****************************************************************************************************************************************************

***********************************
*VARIABLE DESCRIPTIONS*
***********************************

VARIABLE LABELS
pupils 'level-1 identifier'
classes 'level-2 identifier'
gpa 'level-1 predictor variable'
teacher_fan 'level-2 predictor variable'
bieber 'binary outcome variable'.
EXECUTE.

VALUE LABELS teacher_fan 0 'not a belieber' 1 'is a belieber'.
VALUE LABELS bieber 0 'does not own the album' 1 'owns the album'.
EXECUTE.

***********************************************************************************************
*PRELIMINARY PHASE: Centering Variables (corresponding to Sub-Appendix A)*
***********************************************************************************************

*Below are the commands to grand-mean center "gpa" (i.e., subtracting the general mean of the predictor variable).
*The new variable is "gpa_gmc".
AGGREGATE
  /OUTFILE=* MODE=ADDVARIABLES
  /BREAK=
  /grand_mean_gpa=MEAN(gpa).
COMPUTE gpa_gmc=gpa - grand_mean_gpa.
EXECUTE.
DELETE VARIABLES grand_mean_gpa.

*Below are the commands to cluster-mean center "gpa" (i.e., subtracting the class-specific mean of the predictor variable). 
*The new variable is "gpa_cmc".
AGGREGATE
  /OUTFILE=* MODE=ADDVARIABLES
  /BREAK=classes
  /cluster_mean_gpa=MEAN(gpa).
COMPUTE gpa_cmc=gpa - cluster_mean_gpa.
EXECUTE.
DELETE VARIABLES cluster_mean_gpa.

*Below is the command to center "teacher_fan".
RECODE teacher_fan (0=-0.5) (1=0.5) INTO teacher_fan_c.
EXECUTE.

*************************************************************************************
*STEP #1: Building the Empty Model (corresponding to Sub-Appendix B)*
*************************************************************************************

*Below are the commands to run an empty model, that is, a model containing no predictors, 
//and calculate the intraclass correlation coefficient (ICC; the degree of homogeneity of the outcome within clusters).
GENLINMIXED
  /DATA_STRUCTURE SUBJECTS=classes
  /FIELDS TARGET= bieber TRIALS=NONE OFFSET=NONE
  /TARGET_OPTIONS DISTRIBUTION=MULTINOMIAL LINK=LOGIT
  /FIXED USE_INTERCEPT=TRUE
  /RANDOM USE_INTERCEPT=TRUE SUBJECTS=classes COVARIANCE_TYPE=VARIANCE_COMPONENTS 
  /BUILD_OPTIONS TARGET_CATEGORY_ORDER=DESCENDING INPUTS_CATEGORY_ORDER=ASCENDING MAX_ITERATIONS=100 
    CONFIDENCE_LEVEL=95 DF_METHOD=RESIDUAL COVB=MODEL PCONVERGE=0.000001(ABSOLUTE) SCORING=0 
    SINGULAR=0.000000000001
  /EMMEANS_OPTIONS SCALE=ORIGINAL PADJUST=LSD.

*Notes: 
//By default, SPSS does not use “1” as the target category; 
//TARGET_CATEGORY_ORDER=ASCENDING allows you to predict the odds that the outcome variable equals “1” instead of “0” (rather than the reverse); 
//SPSS does not provide you with the ICC; 
//the ICC has to be calculated using the formula given in the article

**************
*RESULTS*
**************
*N = 2000 pupils are nested in K = 100 classes (n = 20 pupils per classes);
*The fixed intercept is B00 = 0.10; 
*The random intercept variance (i.e., the level-2 residual) var(u0j) is [var(intercept)]² = 1.13² = 1.28 (in the seventh windows of the Model Viewer “Covariance parameters,” Block 1)
*The ICC has to be calculted by hand, i.e., var(u0j) + [var(u0j) + (π2 / 3)] = 1.28 + (1.28 + 3.29) = .28
//This indicates that 28% of the chance of owning an album is explained by between-classroom differences 

*********************************************************************************************
*STEP #2: Building the Intermediate Models (corresponding to Sub-Appendix C)*
*********************************************************************************************

*If you focus on the between-observation effect of the (level-1) variable, you can use the grand-mean centered variable ("gap_gmc"). 
*If you focus on the within-cluster effect, use the cluster-mean centered variable ("gap_cmc"). 
*We use the cluster-mean centered variable herein.

*Below is the command to run the constrained intermediate model (CIM; 
//the model contains all level-1 variables, all level-2 variables well as all intra-level interactions).
GENLINMIXED
  /DATA_STRUCTURE SUBJECTS=classes  /FIELDS TARGET= bieber TRIALS=NONE OFFSET=NONE
  /TARGET_OPTIONS REFERENCE=1 DISTRIBUTION=MULTINOMIAL LINK=LOGIT
  /FIXED EFFECTS=gpa_cmc teacher_fan_c
USE_INTERCEPT=TRUE
  /RANDOM USE_INTERCEPT=TRUE SUBJECTS=classes COVARIANCE_TYPE=VARIANCE_COMPONENTS 
  /BUILD_OPTIONS TARGET_CATEGORY_ORDER=DESCENDING INPUTS_CATEGORY_ORDER=ASCENDING MAX_ITERATIONS=100 
    CONFIDENCE_LEVEL=95 DF_METHOD=RESIDUAL COVB=MODEL PCONVERGE=0.000001(ABSOLUTE) SCORING=0 
    SINGULAR=0.000000000001
  /EMMEANS_OPTIONS SCALE=ORIGINAL PADJUST=LSD.

*Below is the command to run the augmented intermediate model (AIM; 
//the model similar to the constrained intermediate model with the exception that it includes the random slope term of “gap_cmc”).
GENLINMIXED
  /DATA_STRUCTURE SUBJECTS=classes  /FIELDS TARGET= bieber TRIALS=NONE OFFSET=NONE
  /TARGET_OPTIONS REFERENCE=1 DISTRIBUTION=MULTINOMIAL LINK=LOGIT
  /FIXED EFFECTS=gpa_cmc teacher_fan_c
USE_INTERCEPT=TRUE
  /RANDOM USE_INTERCEPT=TRUE SUBJECTS=classes COVARIANCE_TYPE=VARIANCE_COMPONENTS 
  /RANDOM EFFECTS=gpa_cmc USE_INTERCEPT=FALSE COVARIANCE_TYPE=VARIANCE_COMPONENTS 
  /BUILD_OPTIONS TARGET_CATEGORY_ORDER=DESCENDING INPUTS_CATEGORY_ORDER=ASCENDING MAX_ITERATIONS=100 
    CONFIDENCE_LEVEL=95 DF_METHOD=RESIDUAL COVB=MODEL PCONVERGE=0.000001(ABSOLUTE) SCORING=0 
    SINGULAR=0.000000000001
  /EMMEANS_OPTIONS SCALE=ORIGINAL PADJUST=LSD.

*Note:
//When a multilevel logistic regression model includes a random slope variance component (which is the case in the AIM), SPSS will often—if not always—fail to estimate parameters;
//In the present situation, we cannot achieve STEP #2 and the three-step procedure cannot be complete;
//In SPSS, "-2 log pseudo likelihood" should be taken as the model deviance statistics.

**************
*RESULTS*
**************
*The model deviance statistics, "-2 log pseudo likelihood" (in the first window of the Model Viewer). can be used to compare the CIM and AIM ;
//the likelihood-ratio test LR X(1)² would have to be calculted by hand using the formula provided in the article.
*However, since the SPSS fails to estimate the AIM parameters, we CANNOT achieve STEP #2 (and we should normally not go to STEP #3).

***********************************************************************************
*STEP #3: Building the Final Model (corresponding to Sub-Appendix D)*
***********************************************************************************

*Below are the commands to compute the "gap_cmc" * "teacher_fan_c" interaction
//and to run the final model (adding the cross-level interaction(s)).
COMPUTE gpa_cmcXteacher_fan_c=gpa_cmc*teacher_fan_c.
GENLINMIXED
  /DATA_STRUCTURE SUBJECTS=classes  /FIELDS TARGET= bieber TRIALS=NONE OFFSET=NONE
  /TARGET_OPTIONS REFERENCE=1 DISTRIBUTION=MULTINOMIAL LINK=LOGIT
  /FIXED EFFECTS=gpa_cmc teacher_fan_c gpa_cmcXteacher_fan_c
USE_INTERCEPT=TRUE
  /RANDOM USE_INTERCEPT=TRUE SUBJECTS=classes COVARIANCE_TYPE=VARIANCE_COMPONENTS 
  /BUILD_OPTIONS TARGET_CATEGORY_ORDER=DESCENDING INPUTS_CATEGORY_ORDER=ASCENDING MAX_ITERATIONS=100 
    CONFIDENCE_LEVEL=95 DF_METHOD=RESIDUAL COVB=MODEL PCONVERGE=0.000001(ABSOLUTE) SCORING=0 
    SINGULAR=0.000000000001
  /EMMEANS_OPTIONS SCALE=ORIGINAL PADJUST=LSD.

*Note:
//we have decided here not to keep the random slope component of “gap_cmc”

*Below are the commands to calculate the first simple slope
//namely the effect of “teacher_fan_c” when “gap_cmc” is low (-1 SD).
AGGREGATE
  /OUTFILE=* MODE=ADDVARIABLES
  /BREAK=
  /sd_gpa_cmc=SD(gpa_cmc).
COMPUTE gpa_m1SD=gpa_cmc + sd_gpa_cmc.
COMPUTE gpa_m1SDXteacher_fan_c=gpa_m1SD*teacher_fan_c.
GENLINMIXED
  /DATA_STRUCTURE SUBJECTS=classes  /FIELDS TARGET= bieber TRIALS=NONE OFFSET=NONE
  /TARGET_OPTIONS REFERENCE=1 DISTRIBUTION=MULTINOMIAL LINK=LOGIT
  /FIXED EFFECTS=gpa_m1SD teacher_fan_c gpa_m1SDXteacher_fan_c
USE_INTERCEPT=TRUE
  /RANDOM USE_INTERCEPT=TRUE SUBJECTS=classes COVARIANCE_TYPE=VARIANCE_COMPONENTS 
  /BUILD_OPTIONS TARGET_CATEGORY_ORDER=DESCENDING INPUTS_CATEGORY_ORDER=ASCENDING MAX_ITERATIONS=100 
    CONFIDENCE_LEVEL=95 DF_METHOD=RESIDUAL COVB=MODEL PCONVERGE=0.000001(ABSOLUTE) SCORING=0 
    SINGULAR=0.000000000001
  /EMMEANS_OPTIONS SCALE=ORIGINAL PADJUST=LSD.

*Below are the commands to calculate the second simple slope
//namely the effect of “teacher_fan_c” when “gap_cmc” is low (-1 SD).
COMPUTE gpa_p1SD=gpa_cmc - sd_gpa_cmc.
COMPUTE gpa_p1SDXteacher_fan_c=gpa_p1SD*teacher_fan_c.
GENLINMIXED
  /DATA_STRUCTURE SUBJECTS=classes  /FIELDS TARGET= bieber TRIALS=NONE OFFSET=NONE
  /TARGET_OPTIONS REFERENCE=1 DISTRIBUTION=MULTINOMIAL LINK=LOGIT
  /FIXED EFFECTS=gpa_p1SD teacher_fan_c gpa_p1SDXteacher_fan_c
USE_INTERCEPT=TRUE
  /RANDOM USE_INTERCEPT=TRUE SUBJECTS=classes COVARIANCE_TYPE=VARIANCE_COMPONENTS 
  /BUILD_OPTIONS TARGET_CATEGORY_ORDER=DESCENDING INPUTS_CATEGORY_ORDER=ASCENDING MAX_ITERATIONS=100 
    CONFIDENCE_LEVEL=95 DF_METHOD=RESIDUAL COVB=MODEL PCONVERGE=0.000001(ABSOLUTE) SCORING=0 
    SINGULAR=0.000000000001
  /EMMEANS_OPTIONS SCALE=ORIGINAL PADJUST=LSD.
EXECUTE.

*Below are the command to compare the coefficient estimates obtained in the final model,
//with (GENLINMIXED) or without (LOGISTIC REGRESSINO) the use of multilevel modelling.
GENLINMIXED
  /DATA_STRUCTURE SUBJECTS=classes  /FIELDS TARGET= bieber TRIALS=NONE OFFSET=NONE
  /TARGET_OPTIONS REFERENCE=1 DISTRIBUTION=MULTINOMIAL LINK=LOGIT
  /FIXED EFFECTS=gpa_cmc teacher_fan_c gpa_cmcXteacher_fan_c
USE_INTERCEPT=TRUE
  /RANDOM USE_INTERCEPT=TRUE SUBJECTS=classes COVARIANCE_TYPE=VARIANCE_COMPONENTS 
  /RANDOM EFFECTS=USE_INTERCEPT=FALSE COVARIANCE_TYPE=VARIANCE_COMPONENTS 
  /BUILD_OPTIONS TARGET_CATEGORY_ORDER=DESCENDING INPUTS_CATEGORY_ORDER=ASCENDING MAX_ITERATIONS=100 
    CONFIDENCE_LEVEL=95 DF_METHOD=RESIDUAL COVB=MODEL PCONVERGE=0.000001(ABSOLUTE) SCORING=0 
    SINGULAR=0.000000000001
  /EMMEANS_OPTIONS SCALE=ORIGINAL PADJUST=LSD.
LOGISTIC REGRESSION VARIABLES bieber
  /METHOD=ENTER gpa_cmc teacher_fan_c gpa_cmcXteacher_fan_c 
  /CRITERIA=PIN(.05) POUT(.10) ITERATE(20) CUT(.5).