****************************************************************************************************************************
*Keep Calm and Learn Multilevel Logistic Modeling: A Simplified Three-Step Procedure for Beginners Using SPSS, Stata, and R*
****************************************************************************************************************************

*if you see a problem with the content of this do-file, please contact nicolas.sommet@unil.ch

***********************
*VARIABLE DESCRIPTIONS*
***********************

label variable pupils "level-1 identifier"
label variable classes "level-2 identifier"
label variable gpa "level-1 predictor variable"
label variable teacher_fan "level-2 predictor variable"
label variable bieber "binary outcome variable"

**************************************************************************
*PRELIMINARY PHASE: Centering Variables (corresponding to Sub-Appendix A)*
**************************************************************************

*Below are the commands to grand-mean center “gpa” (i.e., subtracting the general mean of the predictor variable).
*The new variable is “gpa_gmc”.
egen grand_mean_gpa = mean(gpa)
gen gpa_gmc = gpa - grand_mean_gpa
drop grand_mean_gpa

*Below are the commands to cluster-mean center “gpa” (i.e., subtracting the class-specific mean of the predictor variable). 
*The new variable is “gpa_cmc”.
sort classes
by classes: egen cluster_mean_gpa = mean(gpa)
gen gpa_cmc = gpa - cluster_mean_gpa
drop cluster_mean_gpa

*Below is the command to center “teacher_fan”
recode teacher_fan (0 = -0.5) (1 = 0.5), gen(teacher_fan_c)

*********************************************************************
*STEP #1: Building the Empty Model (corresponding to Sub-Appendix B)*
*********************************************************************

*Below are the commands to run an empty model, that is, a model containing no predictors, 
//and calculate the intraclass correlation coefficient (ICC; the degree of homogeneity of the outcome within clusters).
xtmelogit bieber || classes:, var
estat icc

*********
*RESULTS*
*********
*N = 2000 pupils are nested in K = 100 classes (n = 20 pupils per classes);
*The fixed intercept is B00 = 0.10; 
*The random intercept variance (i.e., the level-2 residual) is var(u0j) = 1.27;
*The intraclass correlation coefficient is ICC = .28; 
//This indicates that 28% of the chance of owning an album is explained by between-classroom differences 

*****************************************************************************
*STEP #2: Building the Intermediate Models (corresponding to Sub-Appendix C)*
*****************************************************************************

*If you focus on the between-observation effect of the (level-1) variable, you can use the grand-mean centered variable (“gpa_gmc”). 
*If you focus on the within-cluster effect, use the cluster-mean centered variable (“gpa_cmc”). 
*We use the cluster-mean centered variable herein.

*Below are the commands to run the constrained intermediate model (CIM; 
//the model contains all level-1 variables, all level-2 variables well as all intra-level interactions) 
//and store its deviance (the smaller the deviance, the better the fit).
xtmelogit bieber gpa_cmc teacher_fan_c || classes:, var
display "FYI: The deviance of the CIM is:" -2*e(ll)
estimate store CIM

*Below are the commands to run the augmented intermediate model (AIM; 
//the model similar to the constrained intermediate model with the exception that it includes the random slope term of “gpa_cmc”) 
//and store its deviance.
xtmelogit bieber gpa_cmc teacher_fan_c || classes: gpa_cmc, var
display "FYI: The deviance of the AIM is " -2*e(ll)
estimate store AIM

*Below is the command to determine whether including the random slope of “gpa_cmc” improves the model. 
*The software performs a likelihood-ratio test LR X(1)²,  comparing the deviance of the CIM with the deviance of the AIM.
lrtest CIM AIM

*********
*RESULTS*
*********
*The random slope component is var(u1j) = 0.83 (corresponding to the variation of the effect of “gpa_cmc” from one class to another);
*The likelihood-ratio test is LR X(1)² = 29.04, p < .001.
*Thus, allowing the effect of “gpa_cmc” to vary between classes improves the fit and it is better to take the random slope into account. 

*********************************************************************
*STEP #3: Building the Final Model (corresponding to Sub-Appendix D)*
*********************************************************************

*Below is the command to run the final model (adding the cross-level interaction(s)). 
xtmelogit bieber c.gpa_cmc##c.teacher_fan_c || classes: gpa_cmc, or var

*Note:
//we have decided here to keep the random slope component of “gpa_cmc”

*Below are the commands to calculate the first simple slope
//namely the effect of “teacher_fan_c” when “gpa_cmc” is low (-1 SD).
egen sd_gpa_cmc = sd(gpa_cmc)
gen gpa_m1SD = gpa_cmc + sd_gpa_cmc
xtmelogit bieber c.gpa_m1SD##c.teacher_fan_c || classes: gpa_cmc, or var

*Below are the commands to calculate the second simple slope
//namely the effect of “teacher_fan_c” when “gpa_cmc” is high (+1 SD).
gen gpa_p1SD = gpa_cmc - sd_gpa_cmc
xtmelogit bieber c.gpa_p1SD##c.teacher_fan_c || classes: gpa_cmc, or var

*********
*RESULTS*
*********
*Regarding your main effect hypothesis, the effect of “teacher_fan_c” is significant, exp(B01) = OR = 7.50, 95% CI [5.00, 11.25]
//pupils whose teacher is a belieber have 7.50 times more chance of owning Justin’s album than pupils whose classroom teacher is not a fan a Justin Bieber 
*Regarding your interaction hypothesis, the interaction between “teacher_fan_c” and “gpa_cmc” is significant, exp(B11) = OR = 3.01, 95% CI [1.86, 4.86]
//simple slope analysis show that for the lowest achievers of their classroom (-1 SD), 
//having a teacher who is a belieber results in 3.50 times more chance of owning Justin’s last album, exp(B01) = OR = 3.47. 95% CI [2.13, 5.64]
//...and for the higher achievers of their classroom (+1 SD), 
//having a teacher who is a belieber results in 16.20 times more chance of owning Justin’s last album, exp(B01) = OR = 16.20 95% CI [9.21, 28.49]

*Below are the commands to compare the coefficient estimates obtained in the final model,
//with (xtmelogit) or without (logit) the use of multilevel modelling.
xtmelogit bieber c.gpa_cmc##c.teacher_fan_c || classes: gpa_cmc, var
logit bieber c.gpa_cmc##c.teacher_fan_c
